@@ -1,71 +1,33 @@
-
-import pgwidget_pkg.pgwidget.engine as engine
-
-import pygame
-
-
-
 import sys
-
 import os
-
+import pygame
 import pandas as pd
 from multinherit.multinherit import multi_super
-
+import tkinter as tk
+import abc
+import inspect
 from helper import c
-
 import datetime
+import threading
+import math
+
+if sys.platform == "darwin":
+    import pygame as engine
+else:
+    import pgwidget.engine as engine
 
 """TKINTER PART"""
-
-import tkinter as tk
-
-import abc
-#from tkinter import LEFT
-
-
-
-import doxyplot.doxyplot_core as dp
-
-
-
-
-
-
-
 root = tk.Tk()
 root.withdraw()
-
-
-
-
-
-"""
-
-WINDOW_SIZE=[1366,768]
-
-embed = tk.Frame(root, width = WINDOW_SIZE[0], height = WINDOW_SIZE[1]) #creates embed frame for pygame window
-embed.grid(columnspan = (600), rowspan = 500) # Adds grid
-#embed.pack(side = LEFT) #packs window to the left
-
-#buttonwin = tk.Frame(root, width = 75, height = 500)
-#buttonwin.pack(side = LEFT)
-
-os.environ['SDL_WINDOWID'] = str(embed.winfo_id())
-os.environ['SDL_VIDEODRIVER'] = 'windib'
-"""
-
 """TKINTER END PART"""
 
-
 def initialize_pg(is_resizable=True, bg_color=(150,150,150), window_size=[1366,768], caption="", window_icon_path=None):
     
     if is_resizable:
         screen = engine.display.set_mode(window_size,engine.RESIZABLE)
     else:
         screen = engine.display.set_mode(window_size)
-       
-        
+              
     engine.init()
     engine.display.set_caption(caption)
     #clock=pygame.time.Clock()
@@ -74,12 +36,11 @@ def initialize_pg(is_resizable=True, bg_color=(150,150,150), window_size=[1366,7
         engine.display.flip()
     except AttributeError: #web has no fill function
         print("Screen fill - engine disable")
-     
         
     try:
         window_icon = engine.image.load(window_icon_path) #e.g. 'src//png//forloop_icon.png'
         pygame.display.set_icon(window_icon)
-    except FileNotFoundError:
+    except (FileNotFoundError,TypeError):
         print("Warning: Icon (icon.png) not found, skipped")
         pass
     
@@ -87,74 +48,8 @@ def initialize_pg(is_resizable=True, bg_color=(150,150,150), window_size=[1366,7
 
 
 #screen=initialize_pg()
-
-"""TKINTER PART"""
-
-"""
-def open_file(): 
-    file = askopenfile(mode ='r', filetypes =[('Python Files', '*.py')]) 
-    filename=""
-    if file is not None: 
-        content = file.read() 
-        #print(content) 
-        filename=file.name
-        
-    return(filename)
-
-
-
-def run_script_from_file(): 
-    file = askopenfile(mode ='r', filetypes =[('Python Files', '*.py')]) 
-    filename=""
-    if file is not None: 
-        content = file.read() 
-        #print(content) 
-        filename=file.name
-    os.system("python "+filename) 
-    #return(filename)
-
-global loaded_filename
-
-def open_xlsx_file(*args): 
-    table1=args[0]
-    file = askopenfile(mode ='r', filetypes =[('Excel files', '*.xlsx')]) 
-    filename=""
-    if file is not None: 
-        #content = file.read() 
-        #print(content) 
-        filename=file.name
-    
-    global loaded_filename
-    loaded_filename=filename
-    df=pd.read_excel(loaded_filename)
-    table1.update_data(df)
-    return(filename)
-
-
-def file_save():
-    global text
-    f = asksaveasfile(mode='w', defaultextension=".txt")
-    if f is None: # asksaveasfile return `None` if dialog closed with "cancel".
-        return
-    text2save = str(text.get(1.0, END)) # starts from `1.0`, not `0.0`
-    f.write(text2save)
-    f.close() # `()` was missing.
-
-
-from tkinter import END
-from tkinter.filedialog import askopenfile,asksaveasfile
-
-#button1 = Button(buttonwin,text = 'Draw',  command=open_file)
-#button1.pack(side=LEFT)
-"""
-
-
-
-
 root.update()
     
-"""TKINTER END PART"""
-
 
 class Label:
     def __init__(self,text,color=(0,0,0),pos=[0,0],relative_pos=[0,0],font_type="Calibri",font_size=14,max_text_length=None,visible=True,shown_text_max_length=1000,is_cursor_drawing=False, is_multiline_label=False):
@@ -171,14 +66,21 @@ class Label:
         self.font_size=font_size
         self.relative_pos=relative_pos
         if "Calibri"==font_type:
-            self.myfont = engine.font.Font("fonts/calibri.ttf",self.font_size)
+            if sys.platform == "darwin" or sys.platform == "linux":
+                self.myfont = engine.font.Font("fonts/Carlito-Regular.ttf",self.font_size)
+            else:    
+                #self.myfont = engine.font.Font("fonts/calibri.ttf",self.font_size) #deprecated
+                self.myfont = engine.font.SysFont(self.font_type, self.font_size)
         elif "Helvetica"==font_type:
-            self.myfont = engine.font.Font("fonts/helvetica.ttf",self.font_size)
+            if sys.platform == "darwin" or sys.platform == "linux":
+                self.myfont = engine.font.Font("fonts/helvetica.ttf",self.font_size)
+            else:    
+                self.myfont = engine.font.SysFont(self.font_type, self.font_size)
         elif ".ttf" in font_type:
             self.myfont = engine.font.Font(font_type,self.font_size)
         else:
             self.myfont = engine.font.SysFont(self.font_type, self.font_size)
-        self.myfont = engine.font.SysFont(self.font_type, self.font_size)
+        
         self.lbl=self.myfont.render(self.text, True, self.color)
         self.visible=visible
         self.visibility_layer=100
@@ -202,7 +104,6 @@ class Label:
 
         self.is_interactive_mode_enabled = True
         
-        
     @property
     def selected(self):
         return(self._selected)
@@ -220,7 +121,10 @@ class Label:
         
     @text.setter
     def text(self,text):
-        
+        if type(text)==str: #handling of NoneType
+            if not self.is_multiline_label:
+                text = text.replace("\n", " ")
+            
         self._text=str(text)
         self.refresh_shown_text()
         self.shown_text_index_offset=len(self._text)-len(self.shown_text)
@@ -257,6 +161,27 @@ class Label:
     def switch_interactive_mode(self):
         self.is_interactive_mode_enabled = not self.is_interactive_mode_enabled    
      
+       
+    def _cut_big_part_of_shown_text(self):
+        ratio = self.max_text_length / self.text_length
+        if ratio < 0.98:  # MAGIC CONSTANT - BETTER CUT BIG PART IN ON SLICE THAN GO ONE AFTER ANOTHER
+            last_char_index = math.floor(ratio * self.text_length)
+
+
+            self.shown_text=self.shown_text[:last_char_index]
+            self.text_length=self.myfont.size(self.shown_text)[0]
+            ratio = self.max_text_length / self.text_length
+        return(ratio)
+        
+    def _cut_shown_text(self):
+        ratio=self._cut_big_part_of_shown_text()
+        if ratio < 1.:
+            while self.text_length>self.max_text_length:
+                if self.selected:
+                    self.shown_text=self.shown_text[1:]
+                else:
+                    self.shown_text=self.shown_text[:-1]
+                self.text_length=self.myfont.size(self.shown_text)[0]
         
     def refresh_shown_text(self):
         if self.is_multiline_label:
@@ -274,15 +199,10 @@ class Label:
     
         self.text_length=self.myfont.size(self.shown_text)[0]
 
-        if not self.is_multiline_label:
+        if not self.is_multiline_label and self.text_length > 0:
             if self.max_text_length is not None:
-                
-                while self.text_length>self.max_text_length:
-                    if self.selected:
-                        self.shown_text=self.shown_text[1:]
-                    else:
-                        self.shown_text=self.shown_text[:-1]  
-                    self.text_length=self.myfont.size(self.shown_text)[0]
+
+                self._cut_shown_text()
         
     def draw(self,screen):
         
@@ -299,7 +219,6 @@ class Label:
                 pixel_length=self.get_text_pixel_length(letter_index=self.highlighted_text_indices[0], letter_row=self.cursor_row_index)
                 pixel_length2=self.get_text_pixel_length(letter_index=self.highlighted_text_indices[1], letter_row=self.cursor_row_index)
                 highlighted_length=abs(pixel_length2-pixel_length)
-                print("HIGHLIGHT",pixel_length,pixel_length2,highlighted_length)
                 
                 engine.draw.rect(screen,(0,0,200),self.cursor_position+[highlighted_length,20])
             try:
@@ -367,7 +286,11 @@ class Label:
         x_offset=pos[0]-self.pos[0]
         y_offset = pos[1] - self.pos[1]
 
-        shown_text_rows = self.shown_text.split('\n')
+        if self.is_multiline_label:
+            shown_text_rows = self.shown_text.split('\n')
+        else:
+            shown_text_rows = [self.shown_text]
+
         row_count = len(shown_text_rows)
         total_shown_text_height = self.myfont.size(self.shown_text)[1]*row_count
 
@@ -386,13 +309,12 @@ class Label:
         # total_shown_text_length=self.myfont.size(self.shown_text)[0]
         total_shown_text_length = self.myfont.size(shown_text_rows[letter_row])[0]
         text_length=0
-        print(text_length,total_shown_text_length)
+        
         if x_offset>total_shown_text_length:
             letter_index=len(self.shown_text) #
         else:
             letter_index=0       
             while text_length<x_offset:        
-                print(text_length,x_offset,total_shown_text_length)
                 text_length=self.get_text_pixel_length(letter_index, letter_row)
                 letter_index+=1
             letter_index-=1 # while loop loops incrementing 1 one more time than it should to check stop criterion - this is correction to reference right index in string
@@ -421,7 +343,6 @@ class Label:
         self.visible = True
         pos=engine.mouse.get_pos()
         
-        print("SHIFT",self.cursor_offset_index,click_with_shift)
         if click_with_shift:
             cursor_offset_index_memory=self.cursor_offset_index
             
@@ -430,9 +351,9 @@ class Label:
         
         
         if self.is_point_in_rectangle(pos) or click_around_label_permitted:
-            self.cursor_row_index = self.shown_cursor_row_index
             self.shown_cursor_offset_index, self.shown_cursor_row_index = self._round_cursor_position_to_nearest_letter(pos)
             self.cursor_offset_index=self.shown_cursor_offset_index+self.shown_text_index_offset
+            self.cursor_row_index = self.shown_cursor_row_index
             
             if cursor_offset_index_memory is not None:
                 self.highlighted_text_indices=sorted([self.cursor_offset_index,cursor_offset_index_memory])
@@ -448,7 +369,10 @@ class Label:
             
     def on_key_down(self,event):
         if (self.cursor_offset_index is not None or self.cursor_row_index is not None) and self.is_interactive_mode_enabled:
-            shown_text_rows = self.shown_text.split('\n')
+            if self.is_multiline_label:
+                shown_text_rows = self.shown_text.split('\n')
+            else:
+                shown_text_rows = [self.shown_text]
             
             if event.key == engine.K_RIGHT:
                 if self.is_multiline_label:
@@ -481,8 +405,8 @@ class Label:
                     self._move_one_row_down()
         
     def _move_one_row_up(self):
-            self.shown_cursor_row_index = max(self.shown_cursor_row_index - 1, 0)
-            self.cursor_row_index = self.shown_cursor_row_index
+        self.shown_cursor_row_index = max(self.shown_cursor_row_index - 1, 0)
+        self.cursor_row_index = self.shown_cursor_row_index
 
     def _move_one_row_down(self):
         shown_text_rows = self.shown_text.split('\n')
@@ -493,28 +417,14 @@ class Label:
     def get_pixel_length(self):
         text_pixels=self.myfont.size(self.text)[0]
         return(text_pixels)
-        
-"""          
-def refresh(pgwidgets):
-    #TO BE DEPRECATED
-    screen.fill(bg_color)
-        
-    for j,widget_type in enumerate(pgwidgets):
-        for i,widget in enumerate(widget_type.elements):
-            widget.draw(screen)
-"""            
 
- 
-         
+
 class CollidableComponent(abc.ABC):    
     def is_collided(self,rects):
         is_collided_at_least_once=False
         collision_entry = None
         for i in range(len(rects)):
-            if rects[i]!=self:
-                
-                #print("collision")
-                #print(rects[i].pos,self.pos)
+            if rects[i]!=self:      
                 collision=False
                 if self.is_point_in_rectangle([rects[i].pos[0],rects[i].pos[1]]):
                     collision=True
@@ -586,7 +496,8 @@ class SelectableComponent(abc.ABC):
 class ComponentContainingLabels(abc.ABC):
     @abc.abstractmethod
     def __init__(self):
-        self.labels=[]
+        if not hasattr(self, 'labels'):
+            self.labels=[]
         
         
     def draw_labels(self,screen):
@@ -666,6 +577,16 @@ class Scrollbar(DraggableRect): #ImprovedDraggableRect
         self.scrollbar_handle=ScrollbarHandle([pos[0],pos[1]+15],[size[0],(size[1]-30)],self.percentage)
         self.max_handle_offset=(1-self.percentage)*(size[1]-30)
         self.scrollbar_handle_ratio=0
+
+    def set_position(self, pos):
+        self.pos = pos
+        self.scrollbar_handle.pos = [self.pos[0],self.pos[1]+15]
+
+    def set_size(self, size):
+        ratio = size[1]/self.size[1]
+        self.size = size
+        self.scrollbar_handle.size = [size[0],self.scrollbar_handle.size[1] * ratio]
+        self.max_handle_offset=(1-self.percentage)*(size[1]-30)
     
     def calculate_handle_offset(self): 
         offset=self.scrollbar_handle.pos[1]-self.pos[1]-15
@@ -676,7 +597,7 @@ class Scrollbar(DraggableRect): #ImprovedDraggableRect
     def calculate_handle_ratio_position(self):
         offset=self.calculate_handle_offset()
         self.scrollbar_handle_ratio=offset/self.max_handle_offset
-        print(self.scrollbar_handle_ratio)
+        
         return(self.scrollbar_handle_ratio)
         
     def draw(self,screen):
@@ -701,16 +622,33 @@ class Scrollbar(DraggableRect): #ImprovedDraggableRect
         
         if pos[1]>self.pos[1]+self.size[1]-15: #upper arrow
             self.shift_scrollbar(10)
+
+    def move_up(self):
+        self.scrollbar_handle.pos[1] = self.pos[1] + 15
+        self.calculate_handle_ratio_position()
+
+
+    def move_down(self):
+        self.scrollbar_handle.pos[1] = self.pos[1] + self.size[1] - self.scrollbar_handle.size[1] -15
+        self.calculate_handle_ratio_position()
+
+
             
     def shift_scrollbar(self,pixel_offset):
-        print(pixel_offset)
+        
         self.scrollbar_handle.pos[1]+=pixel_offset
         #check if valid move
-        if self.calculate_handle_ratio_position()<=1 and self.calculate_handle_ratio_position()>=0:
-            pass
-            #Valid
-        else:
-            self.scrollbar_handle.pos[1]-=pixel_offset #Revert back
+        if self.calculate_handle_ratio_position() > 1:
+            self.move_down()
+        if self.calculate_handle_ratio_position() < 0:
+            self.move_up()
+        self.calculate_handle_ratio_position()
+
+        # if self.calculate_handle_ratio_position()<=1 and self.calculate_handle_ratio_position()>=0:
+        #     pass
+        #     #Valid
+        # else:
+        #     self.scrollbar_handle.pos[1]-=pixel_offset #Revert back
   
                 
     def on_unclick(self):
@@ -718,7 +656,6 @@ class Scrollbar(DraggableRect): #ImprovedDraggableRect
         self.is_clicked=False
                 
     def on_drag(self,offset_pos_y):
-        print(offset_pos_y,engine.mouse.get_pos())
         
         self.scrollbar_handle.pos=[self.scrollbar_handle.pos[0],offset_pos_y]
         self.calculate_handle_ratio_position()
@@ -752,10 +689,10 @@ class ScrollbarHandle(DraggableRect): #ImprovedDraggableRect
 
 
 class ScrollableComponent:
-    def __init__(self,pos,size,horizontal_offset=-15): 
-        self.scrollbar=Scrollbar([pos[0]+size[0]+horizontal_offset,pos[1]],[15,size[1]],0.3)
+    def __init__(self,pos,size,horizontal_offset=-15, scrollbar_width=15):
+        self.scrollbar=Scrollbar([pos[0]+size[0]+horizontal_offset,pos[1]],[scrollbar_width,size[1]],0.3)
         self.total_height=size[1]/self.scrollbar.percentage
-        self.handle_offset=0 
+        self.handle_offset=0
         
        
     def draw(self,screen):
@@ -763,7 +700,6 @@ class ScrollableComponent:
  
         
     def on_click(self):
-        print("Scrollable component on click")
         pos=engine.mouse.get_pos()
         if self.scrollbar.is_point_in_rectangle(pos):
             self.scrollbar.on_click()
@@ -772,7 +708,6 @@ class ScrollableComponent:
         
     def on_drag(self,offset_pos_y):
         pos=engine.mouse.get_pos()
-        print("SCROLLABLE",pos,offset_pos_y)
         if self.scrollbar.is_clicked: 
             self.scrollbar.on_drag(offset_pos_y) 
             self.handle_offset=self.scrollbar.calculate_handle_offset()
@@ -794,14 +729,15 @@ class Grid(ScrollableComponent):
         self.has_scrollbar=True
         self.camera_row_offset=0
         self.camera_col_offset=0
-        
-        
-        self.table_size=[(self.cell_size[0]+self.margin)*self.cols+int(self.has_scrollbar)*15,(self.cell_size[1]+self.margin)*(self.rows+1)]
+
+        scrollbar_width = 15
+
+        self.table_size=[(self.cell_size[0]+self.margin)*self.cols+int(self.has_scrollbar)*scrollbar_width,(self.cell_size[1]+self.margin)*(self.rows+1)]
         self.frame_cell=Cell(self.pos,self.table_size,self.frame_cell_color)
         self.selected_cell_index=None
         if type(self)==Grid:
             self._initialize_cells()
-        super().__init__(pos,self.table_size,scrollbar_horizontal_offset)
+        super().__init__(pos,self.table_size,scrollbar_horizontal_offset, scrollbar_width=scrollbar_width)
         
     
     
@@ -812,6 +748,7 @@ class Grid(ScrollableComponent):
             for i in range(self.rows):    
                 new_pos=[self.pos[0]+j*self.cell_size[0]+j*self.margin,self.pos[1]+i*self.cell_size[1]+i*self.margin]
                 self.table_cells.append(Cell(new_pos,self.cell_size,(255,255,255),coor=[i,j]))
+
             
  
     def is_point_in_rectangle(self,pos):
@@ -835,7 +772,36 @@ class Grid(ScrollableComponent):
         row_index=index%(self.rows+1)
         col_index=index//(self.rows+1)
         return(row_index,col_index)
-    
+
+    def resize_table_by_rows_and_cols(self, rows, cols):
+        #will cut/add cols to the right and cut/add rows to the bottom
+
+
+        self.table_cells = []
+        self.cols = cols
+        self.rows = rows
+
+        self._initialize_cells()
+
+
+        self.table_size=[(self.cell_size[0]+self.margin)*self.cols+int(self.has_scrollbar)*self.scrollbar.size[0],(self.cell_size[1]+self.margin)*(self.rows+1)]
+
+        self.selected_cell_index = None
+
+        self.scrollbar.set_position(pos=[self.pos[0]+self.table_size[0] - self.scrollbar.size[0], self.pos[1]])
+        self.scrollbar.set_size(size=[self.scrollbar.size[0],self.table_size[1]])
+
+        self.frame_cell.size = self.table_size
+        # self.df_
+
+        # self.scrollbar.pos = [self.pos[0]+self.table_size[0] - 15, self.pos[1]]
+
+
+    def resize_table_by_size(self, size):
+        new_row_size = size[0]//(self.cell_size[0] + self.margin)
+        new_col_size = size[1]//(self.cell_size[1] + self.margin)
+        self.resize_table_by_rows_and_cols(new_col_size, new_row_size)
+
     
     def find_cell_index(self,row,col):
         """Assumes rectangular shape of cells"""
@@ -843,7 +809,6 @@ class Grid(ScrollableComponent):
         return(index)
             
     def deselect_all_cells(self):
-        #print("DESELECTING")
         for index,cell in enumerate(self.table_cells):
             self.table_cells[index].selected=False #redundant at the moment
             self.table_cells[index].label.selected=False
@@ -852,7 +817,6 @@ class Grid(ScrollableComponent):
             
     
     def select_cell(self,selected_cell_index):
-        #print("SELECT CELL",selected_cell_index)
         self.selected_cell_index=selected_cell_index
         #is_outside_index=self.selected_cell_index>=len(self.table_cells)
         #if is_outside_index:
@@ -873,27 +837,38 @@ class Grid(ScrollableComponent):
     def move_selected(self,direction):
         if self.selected_cell_index is not None:
             i,j=self.table_cells[self.selected_cell_index].coor
-            #print(i,j)
-            if direction==1:
-                target_coordinates=(i,j+1) #right
+            if direction==1: #right
+                if j==self.cols-1 and i==self.rows: #last cell in whole grid -> don't move
+                    target_coordinates=None
+                elif j==self.cols-1:
+                    target_coordinates=(i+1, 0) #last column -> go to next row to first col
+                else:
+                    target_coordinates=(i,j+1)
+
             if direction==2:
                 target_coordinates=(i-1,j) #up
+
             if direction==3:
                 target_coordinates=(i,j-1) #left
+
             if direction==4:
-                target_coordinates=(i+1,j) #down
-            target_cell_index=self.find_cell_index(*target_coordinates) 
-            if target_cell_index is not None:
-                self.deselect_all_cells()
-                self.select_cell(target_cell_index)
-            else:
-                print("Move cam, branch")
-                self.move_camera(*target_coordinates)
+                if j==self.cols-1 and i==self.rows: #last cell in whole grid -> don't move
+                    target_coordinates=None
+                else:
+                    target_coordinates=(i+1,j) #down
+                    
+            if target_coordinates is not None:
+                target_cell_index=self.find_cell_index(*target_coordinates) 
+                if target_cell_index is not None:
+                    self.deselect_all_cells()
+                    self.select_cell(target_cell_index)
+                else:
+                    print("Move cam, branch")
+                    self.move_camera(*target_coordinates)
            
  
         
     def draw(self,screen):
-        
         self.frame_cell.draw(screen)
         
         for i,cell in enumerate(self.table_cells):
@@ -944,10 +919,8 @@ class Grid(ScrollableComponent):
         self.camera_col_offset #0
         self.rows #25
         self.cols #16
-        #print(target_row_index,self.rows)
         if target_row_index>=self.rows+1: #+1 for header row
             
-            #print("Move cam, branch2")
             self.camera_row_offset+=1
             self.update_data(self.df)
 
@@ -1045,8 +1018,85 @@ class Table(Grid):
         
        
 
+    def resize_table_by_rows_and_cols(self, rows, cols):
+        self.cols = cols
+        self._init_col_width_dict()
+        super().resize_table_by_rows_and_cols(rows, cols)
+
+        self.is_ready_for_updating_df_subset = True
+
+
+    def move_selected_right(self, i, j):
+        target_coordinates=(i+(j+1)//self.cols,(j+1)%self.cols)
+        if target_coordinates[0]>=self.rows+1: #+1 because of header row
+            target_coordinates=None
+
+        return target_coordinates
+
+
+    # def move_selected_up_or_down(self, i, j):
+    #     if is_
         
-    
+    def move_selected_down(self, i, j):
+        if i == self.rows: #if last row
+            if (self.rows + self.camera_row_offset) < self.df.shape[0]: #need to check if there are data 'below' – table should be scrolled
+                target_coordinates = (i, j) #if there are, we keep current selected cell and scroll down by one cell
+                self.scrollbar.shift_scrollbar(self.move_scrollbar_by_one_cell())
+
+            self.is_ready_for_updating_df_subset = True
+        else:
+            target_coordinates = (i + 1, j)
+
+        return target_coordinates
+
+    def move_selected_up(self, i, j):
+        if i == 1:
+            if (self.camera_row_offset) > 0:
+                target_coordinates = (i, j)
+                self.scrollbar.shift_scrollbar(-self.move_scrollbar_by_one_cell())
+
+            self.is_ready_for_updating_df_subset = True
+
+        else:
+            target_coordinates = (i - 1, j)
+
+        return target_coordinates
+
+
+    def move_selected(self,direction):
+        if self.df is None:
+            super().move_selected(direction)
+            return
+
+        if self.selected_cell_index is not None:
+            i,j=self.table_cells[self.selected_cell_index].coor
+
+            if direction==1: #right
+                target_coordinates=self.move_selected_right(i, j)
+
+            if direction==2: #up
+                target_coordinates=self.move_selected_up(i, j)
+
+            if direction==3:#left
+                target_coordinates=(i,j-1)
+
+            if direction==4:#down
+                target_coordinates=self.move_selected_down(i, j)
+            print(target_coordinates)
+            if target_coordinates is not None:
+                target_cell_index=self.find_cell_index(*target_coordinates)
+                if target_cell_index is not None:
+                    self.deselect_all_cells()
+                    self.select_cell(target_cell_index)
+                else:
+                    print("Move cam, branch")
+                    self.move_camera(*target_coordinates)
+
+    def move_scrollbar_by_one_cell(self):
+        scrollbar_range = self.scrollbar.size[1] - self.scrollbar.scrollbar_handle.size[1]
+        one_cell_range = scrollbar_range/(self.df.shape[0] - self.rows)
+        self.scrollbar.calculate_handle_ratio_position()
+        return one_cell_range
     
     def show_data_subset(self,df,row_index=0,col_index=0):
         #a=datetime.datetime.now()
@@ -1074,9 +1124,8 @@ class Table(Grid):
         """data layer"""
         
         self.df=df
-        
-        
-        
+
+
         """TODO: Maybe this could be refactored to not update the whole dataframe, just the view and save some computational power"""
         maximal_row_index_of_top_cell=max(len(self.df)-self.rows,0)  #shouldnt be negative
         normalized_scrollbar_handle_ratio=max(min(self.scrollbar.scrollbar_handle_ratio,1),0)
@@ -1101,7 +1150,7 @@ class Table(Grid):
         for j in range(len(column_names)):
             cell_index=self.find_cell_index(0,j)
             if cell_index is not None:
-                self.table_cells[cell_index].label.text=str(column_names[j])     
+                self.table_cells[cell_index].label.text=str(column_names[j])
         for i in range(min(len(list1), rows)):
             b=datetime.datetime.now()
             #print(i,b-a)
@@ -1283,12 +1332,8 @@ class TextContainerRect(DraggableRect,abc.ABC):
         # Entry inherits draw method
 
 
-    def draw(self, screen):
-        super().draw(screen)
-        for i, label in enumerate(self.labels):
-            # print("Drawing",label.text)
-            # print("Drawing",label.shown_text)
-            label.draw(screen)
+    def draw(self, screen, auto_draw_labels:bool=True):
+        super().draw(screen, auto_draw_labels=auto_draw_labels)
 
     def on_click(self, glc, click_with_shift=False):
         # print("TRYING DESELECT")
@@ -1331,15 +1376,10 @@ class Entry(TextContainerRect):
             self.labels[0].text = text
         
 
-    def draw(self, screen):
-        super().draw(screen)
-        for i, label in enumerate(self.labels):
-            # print("Drawing",label.text)
-            # print("Drawing",label.shown_text)
-            label.draw(screen)
+    def draw(self, screen, auto_draw_labels:bool=True):
+        super().draw(screen, auto_draw_labels=auto_draw_labels)
 
     def on_click(self, glc, click_with_shift=False):
-        print("TRYING DESELECT",click_with_shift)
         # glc.table1.deselect_all_cells() #deselect cells in order to be able to write in Entry
         # print("SELECTED_CELL",glc.table1.selected_cell_index)
         self.labels[0].on_click(click_around_label_permitted=True,click_with_shift=click_with_shift)
@@ -1739,6 +1779,33 @@ class ComboBox(DraggableRect):
 
 
 class CascadeMenu(Button):
+    
+    """
+    #Inspiration for cascade menu
+    import tkinter as tk
+    
+    def donothing():
+        pass
+    
+    
+    root = tk.Tk()
+    
+    
+    menubar = tk.Menu(root)
+    filemenu = tk.Menu(menubar, tearoff=0)
+    filemenu.add_command(label="New", command=donothing)
+    filemenu.add_command(label="Open", command=donothing)
+    filemenu.add_command(label="Save", command=donothing)
+    filemenu.add_separator()
+    filemenu.add_command(label="Exit", command=root.quit)
+    menubar.add_cascade(label="File", menu=filemenu)
+    
+    helpmenu = tk.Menu(menubar, tearoff=0)
+    helpmenu.add_command(label="Help Index", command=donothing)
+    helpmenu.add_command(label="About...", command=donothing)
+    menubar.add_cascade(label="Help", menu=helpmenu)
+    """
+    
     def __init__(self,pos,size,text,color=(100,100,100),border_color=(0,0,0)):
         super().__init__(pos,size,text,color=color,border_color=border_color)       
 
@@ -1747,54 +1814,6 @@ class CascadeMenu(Button):
         super().draw(screen)
 
 
-"""
-import tkinter as tk
-
-def donothing():
-    pass
-
-
-root = tk.Tk()
-
-
-menubar = tk.Menu(root)
-filemenu = tk.Menu(menubar, tearoff=0)
-filemenu.add_command(label="New", command=donothing)
-filemenu.add_command(label="Open", command=donothing)
-filemenu.add_command(label="Save", command=donothing)
-filemenu.add_separator()
-filemenu.add_command(label="Exit", command=root.quit)
-menubar.add_cascade(label="File", menu=filemenu)
-
-helpmenu = tk.Menu(menubar, tearoff=0)
-helpmenu.add_command(label="Help Index", command=donothing)
-helpmenu.add_command(label="About...", command=donothing)
-menubar.add_cascade(label="Help", menu=helpmenu)
-
-"""
-
-
-
-class DoxyplotImage(ButtonImage,dp.Doxyplot):  
-    """Not used at the moment"""
-    def __init__(self,pos,size,img=None,is_draggable=True,frame_color=c.frame,relative_pos=[0,0],has_frame=True):
-        multi_super(ButtonImage,self,pos=pos,size=size,img=img,draggable=True)
-        multi_super(dp.Doxyplot,self)
-        #self.selection_count=0
-        
-
-    def plot(self):
-        pass
-    
-    
-    def draw(self,screen):
-        engine.draw.rect(screen,(100,200,100),self.pos+self.size,1)
-        super().draw(screen)
-        
-        
-
-
-
 
 
 
@@ -1806,13 +1825,6 @@ class PgWidget:
         print("Warning: PgWidget class will be deprecated in future versions")
 
 
-
-
-import inspect
-
-
-
-
 class TimeTrigger:
     def __init__(self,frequency,function=lambda *x:None,args=None):
         self.frequency=frequency
@@ -1831,7 +1843,7 @@ class GuiTimeHandler:
     def tick(self):
         self.t=self.t+10
         
-        print(self.t)
+        #print(self.t)
         #print(self.time_triggers)
         #print(self.refresh_period,"PERIOD")
         
@@ -1844,8 +1856,8 @@ class GuiTimeHandler:
             
         if self.t%self.refresh_period==0:
             self.glc.refresh(self.glc.screen)
-            print("REFRESH")
-            print([(x.visible,x.pos) for x in self.glc.rects])
+            #print("REFRESH")
+            #print([(x.visible,x.pos) for x in self.glc.rects])
             """
             for i,rect in enumerate(rects):
                 rect.is_collided()  
@@ -1909,7 +1921,6 @@ class GuiEventHandler:
 
         for i,element in enumerate(self.glc.rects+self.glc.entries+self.glc.tables):
             if element.is_point_in_rectangle(pos):
-                print(element,"CLICKED")
                 #try:
                 element.on_click(self.glc)
                 #except:
@@ -2016,12 +2027,12 @@ class GuiLayoutContext:
         #    print("Screen fill - engine disable")
         
         pgw_widgets=[widget for widget_type in self.pgwidgets for widget in widget_type.elements] #fancy double list comprehension
-        print(pgw_widgets)
+        #print(pgw_widgets)
         for i,shape in enumerate(pgw_widgets):
             if shape.visible:
                 draw_arguments=list(inspect.signature(shape.draw).parameters.keys()) #analyzes if there's glc in the draw function args
                 if "screen" in draw_arguments:
-                    print("SHAPE",shape)
+                    #print("SHAPE",shape)
                     shape.draw(screen) #screen arg - most natural?
                 else:
                     shape.draw()
@@ -2098,20 +2109,6 @@ class GuiLayoutContext:
             self.text = self.text[:-1]
 
 
-
-
-
-
-
-
-
-#import time
-#time.sleep(10)
-#if __name__ == "crypto_dashboard":
-
-
-import threading
-
 def remote_main_program_loop(glc,geh,gth):
     """GuiLayoutContext,GuiEventHandler,GuiTimeHandler"""
     done=False
